unit DebugLogForm;

interface
                   
{$I CompConditionals.inc}
{.$DEFINE CanExport}

uses
  Windows, Messages, SysUtils, Classes, Graphics, Controls, Forms, Dialogs,
  ComCtrls, ExtCtrls, Buttons, StdCtrls, Menus,

  // DebugLog Units
  DLog;

type
  TDebugForm = class(TForm)
    Button1: TButton;
    Button2: TButton;
    DebugList: TListView;
    Toolbar: TPanel;
    ClearLogButton: TBitBtn;
    ExportButton: TBitBtn;
    DebugListPopup: TPopupMenu;
    DeleteMenuItem: TMenuItem;
    ChoicesBox: TComboBox;
    StatusBar: TStatusBar;
    Label1: TLabel;
    Label2: TLabel;
    Label3: TLabel;
    Label4: TLabel;
    Label5: TLabel;
    Label6: TLabel;
    Panel1: TPanel;
    Shape1: TShape;
    PrintButton: TBitBtn;
    procedure DebugListColumnClick(Sender: TObject; Column: TListColumn);
    procedure FormCreate(Sender: TObject);
    procedure DebugListKeyDown(Sender: TObject; var Key: Word;
      Shift: TShiftState);
    procedure DeleteMenuItemClick(Sender: TObject);
    procedure ChoicesBoxChange(Sender: TObject);
    procedure FormShow(Sender: TObject);
    procedure DebugListInsert(Sender: TObject; Item: TListItem);
    procedure ClearLogButtonClick(Sender: TObject);
    procedure ExportButtonClick(Sender: TObject);
    procedure PrintButtonClick(Sender: TObject);
    procedure Button1Click(Sender: TObject);
    procedure Button2Click(Sender: TObject);
  public
    LogComponent: TDebugLog;

    procedure EnableToolbar(Enable: Boolean);
    procedure Clear;
  end;

var
  DebugForm: TDebugForm;

// Global Callback Functions
function CustomSortProc(Item1, Item2: TListItem; ParamSort: integer): integer; stdcall;

implementation

uses ShellAPI
  {$IFDEF CanExport}
     ,EC_LView, EC_Main
  {$ENDIF};

{$R *.DFM}

// *** Global Callback Functions ***
function CustomSortProc(Item1, Item2: TListItem; ParamSort: integer): integer; stdcall;
  function GetText(Item: TListItem): String;
  begin
    with Item do
      if ParamSort = 0 then
        Result := Caption
      else
        if SubItems.Count >= ParamSort then
          Result := SubItems[ParamSort - 1]
        else Result := '';
  end;

  function IsNumberString(S: String): Boolean;
  var
    i: Integer;
  begin
    S := Trim(S);
    Result := True;
    for i := 1 to Length(S) do
      Result := Result and (S[i] in ['0'..'9']);
  end;

begin
 if (ParamSort = 0) or // If First Column,
     (IsNumberString(GetText(Item1)) and IsNumberString(GetText(Item2))) then // or if Both Strings are Numbers, do a number sort
  begin
    Result := Length(GetText(Item1)) - Length(GetText(Item2));
    if Result = 0 then // If they have the same number of digits then,
      Result := lstrcmp(PChar(GetText(Item1)), PChar(GetText(Item2))); // just compare
  end
  else // otherwise, just sort by comparing
    Result := lstrcmp(PChar(GetText(Item1)),
                      PChar(GetText(Item2)));
end;

// *** TDebugForm Methods ***

procedure TDebugForm.EnableToolbar(Enable: Boolean);
begin
  // Enable Toolbar Buttons
  ClearLogButton.Enabled := Enable;
  ExportButton.Enabled := Enable;
  PrintButton.Enabled := Enable;

  // Enable Popup Menu Items
  DeleteMenuItem.Enabled := Enable;
end;

procedure TDebugForm.Clear;
begin
  ClearLogButtonClick(Self);
end;

// *** Delphi IDE-Maintained Methods ***

procedure TDebugForm.DebugListColumnClick(Sender: TObject;
  Column: TListColumn);
begin
  // Sort the Column
  TListView(Sender).CustomSort(@CustomSortProc, Column.Index);

  // Make First Item Visible
  with TListView(Sender) do
    if Items.Count > 0 then
      Items[0].MakeVisible(False);
end;

procedure TDebugForm.FormCreate(Sender: TObject);
begin
  // Move to Top Left Corner
  Left := 0;
  Top  := 0;

 {$IFDEF Delphi4Up}
  with DebugList do
  begin
    RowSelect := True;
    FullDrag  := True;
  end;
 {$ENDIF}
end;

procedure TDebugForm.ClearLogButtonClick(Sender: TObject);
begin
  with DebugList.Items do
  begin
    Screen.Cursor := crHourglass;
    BeginUpdate;
    try
      // Erase Log Entries
      Clear;

      // Remove All 'Items' Columns
      with DebugList.Columns do
        while Count > cTimeStampCol + 2 do
          Items[cTimeStampCol + 2].Free;

      // Clear StatusBar Text
      StatusBar.Panels[1].Text := '';

      // Disable Toolbar
      EnableToolbar(False);

    finally
      EndUpdate;
      Screen.Cursor := crDefault;
    end;
  end;
end;

procedure TDebugForm.DebugListKeyDown(Sender: TObject; var Key: Word;
  Shift: TShiftState);
begin
  if Key = VK_DELETE then
    DeleteMenuItemClick(Sender);    
end;

procedure TDebugForm.DeleteMenuItemClick(Sender: TObject);
begin
  with DebugList do
  try
    Items.BeginUpdate;

    while SelCount > 0 do
      Selected.Delete;
  finally
    Items.EndUpdate;
  end;
end;

procedure TDebugForm.ChoicesBoxChange(Sender: TObject);
begin
  // Trigger Debug Event
  with LogComponent do
    if Assigned(OnDebug) then
    begin
      ClearLogButtonClick(Sender); // clear log
      OnDebug(LogComponent, TComboBox(Sender).ItemIndex);
    end;
end;

procedure TDebugForm.FormShow(Sender: TObject);
const
  cAlreadyShown = 1;
begin
  // Update the Choices ComboBox
  if Tag <> cAlreadyShown then // Only the first time the form is shown
  begin
   with LogComponent do
     if Assigned(Choices) then
       ChoicesBox.Items.Assign(Choices); // Update the Choices Box

   Tag := cAlreadyShown;
  end;
end;

procedure TDebugForm.DebugListInsert(Sender: TObject; Item: TListItem);
const
  cStatusMsg = '%d Log Entries';
begin
  StatusBar.Panels[1].Text := Format(cStatusMsg, [DebugList.Items.Count]);
end;

procedure TDebugForm.ExportButtonClick(Sender: TObject);
{$IFDEF CanExport}
var
  E: TExportListView;
{$ENDIF}
begin
{$IFDEF CanExport}
  E := TExportListView.Create(Self);
  try
    E.Title := cExportTitle;
    E.Options := E.Options - [NumberRows];
    E.Choose;
  finally
    E.Free;
  end;
{$ELSE}
  Panel1.Visible := True;
  Panel1.BringToFront;
{$ENDIF}
end;

procedure TDebugForm.PrintButtonClick(Sender: TObject);
{$IFDEF CanExport}
var
  E: TExportListView;
{$ENDIF}
begin
{$IFDEF CanExport}
  E := TExportListView.Create(Self);
  try
    E.Title := cExportTitle;
    E.Print;
  finally
    E.Free;
  end;
{$ELSE}
  Panel1.Visible := True;
  Panel1.BringToFront;  
{$ENDIF}
end;

procedure TDebugForm.Button1Click(Sender: TObject);
begin
  Panel1.Visible := False;
end;

procedure TDebugForm.Button2Click(Sender: TObject);
begin
  // Visit our Web Page
  ShellExecute(0, PChar('open'), PChar(cComponentsWebSite), PChar(''), PChar(''), SW_SHOWNORMAL);
end;

end.
